@@ -2,7 +2,7 @@
 /**
  * Resource management for the Kolab server.
  *
- * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.4 2009/03/06 08:43:12 wrobel Exp $
+ * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.11 2009-12-03 14:20:55 wrobel Exp $
  *
  * PHP version 4
  *
@@ -42,7 +42,7 @@ define('RM_ITIP_TENTATIVE',                 3);
 /**
  * Provides Kolab resource handling
  *
- * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.4 2009/03/06 08:43:12 wrobel Exp $
+ * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.11 2009-12-03 14:20:55 wrobel Exp $
  *
  * Copyright 2004-2009 Klarälvdalens Datakonsult AB
  *
@@ -211,7 +211,7 @@ class Kolab_Resource
         require_once 'Horde/Kolab/Storage/List.php';
         $list = &Kolab_List::singleton();
         $default = $list->getForeignDefault($id, 'event');
-        if (!$default) {
+        if (!$default || is_a($default, 'PEAR_Error')) {
             $default = &new Kolab_Folder();
             $default->setList($list);
             $default->setName($conf['kolab']['filter']['calendar_store']);
@@ -247,8 +247,12 @@ class Kolab_Resource
         $object['summary'] = $itip->getAttributeDefault('SUMMARY', '');
         $object['location'] = $itip->getAttributeDefault('LOCATION', '');
         $object['body'] = $itip->getAttributeDefault('DESCRIPTION', '');
-        $object['start-date'] = $itip->getAttributeDefault('DTSTART', '');
-        $object['end-date'] = $itip->getAttributeDefault('DTEND', '');
+        $dtend = $itip->getAttributeDefault('DTEND', '');
+        if (is_array($dtend)) {
+            $object['_is_all_day'] = true;
+        }
+        $object['start-date'] = $this->convert2epoch($itip->getAttributeDefault('DTSTART', ''));
+        $object['end-date'] = $this->convert2epoch($dtend);
 
         $attendees = $itip->getAttribute('ATTENDEE');
         if (!is_a( $attendees, 'PEAR_Error')) {
@@ -282,6 +286,20 @@ class Kolab_Resource
                 if (isset($attendees_params[$i]['ROLE'])) {
                     $attendee['role'] = $attendees_params[$i]['ROLE'];
                 }
+
+                if (isset($attendees_params[$i]['PARTSTAT'])) {
+                    $status = strtolower($attendees_params[$i]['PARTSTAT']);
+                    switch ($status) {
+                    case 'needs-action':
+                    case 'delegated':
+                        $attendee['status'] = 'none';
+                        break;
+                    default:
+                        $attendee['status'] = $status;
+                        break;
+                    }
+                }
+
                 $object['attendee'][] = $attendee;
             }
         }
@@ -506,18 +524,21 @@ class Kolab_Resource
                         $duration = $dtend - $dtstart;
                         $events = array();
                         $next_start = $vfbstart;
-                        while ($next = $recurrence->nextActiveRecurrence($next_start)) {
-                            $next_start = $next->timestamp();
-                            if ($next_start < $vfbend) {
-                                $events[$next_start] = $next_start + $duration;
-                            } else {
-                                break;
-                            }
+                        $next = $recurrence->nextActiveRecurrence($vfbstart);
+                        while ($next !== false && $next->compareDate($vfbend) <= 0) {
+                            $next_ts = $next->timestamp();
+                            $events[$next_ts] = $next_ts + $duration;
+                            $next = $recurrence->nextActiveRecurrence(array('year' => $next->year,
+                                                                            'month' => $next->month,
+                                                                            'mday' => $next->mday + 1,
+                                                                            'hour' => $next->hour,
+                                                                            'min' => $next->min,
+                                                                            'sec' => $next->sec));
                         }
                     } else {
                         $events = array($dtstart => $dtend);
                     }
-                    
+
                     foreach ($events as $dtstart => $dtend) {
                         foreach ($busyperiods as $busyfrom => $busyto) {
                             if (empty($busyfrom) && empty($busyto)) {
@@ -546,20 +567,20 @@ class Kolab_Resource
                             break;
                         }
                     }
-                }
 
-                if ($conflict) {
-                    if ($action == RM_ACT_MANUAL_IF_CONFLICTS) {
-                        //sendITipReply(RM_ITIP_TENTATIVE);
-                        Horde::logMessage('Conflict detected; Passing mail through',
-                                          __FILE__, __LINE__, PEAR_LOG_INFO);
-                        return true;
-                    } else if ($action == RM_ACT_REJECT_IF_CONFLICTS) {
-                        Horde::logMessage('Conflict detected; rejecting',
-                                          __FILE__, __LINE__, PEAR_LOG_INFO);
-                        $this->sendITipReply($cn, $id, $itip, RM_ITIP_DECLINE,
-                                             $organiser, $uid, $is_update);
-                        return false;
+                    if ($conflict) {
+                        if ($action == RM_ACT_MANUAL_IF_CONFLICTS) {
+                            //sendITipReply(RM_ITIP_TENTATIVE);
+                            Horde::logMessage('Conflict detected; Passing mail through',
+                                              __FILE__, __LINE__, PEAR_LOG_INFO);
+                            return true;
+                        } else if ($action == RM_ACT_REJECT_IF_CONFLICTS) {
+                            Horde::logMessage('Conflict detected; rejecting',
+                                              __FILE__, __LINE__, PEAR_LOG_INFO);
+                            $this->sendITipReply($cn, $id, $itip, RM_ITIP_DECLINE,
+                                                 $organiser, $uid, $is_update);
+                            return false;
+                        }
                     }
                 }
             }
@@ -668,11 +689,6 @@ class Kolab_Resource
             Horde::logMessage(sprintf('Removing event %s', $uid),
                               __FILE__, __LINE__, PEAR_LOG_INFO);
 
-            /* Ensure that $summary is single byte here, else we get
-             * problems with sprintf()
-             */
-            $summary = String::convertCharset($summary, 'utf-8', 'iso-8859-1');
-
             if (is_a($imap_error, 'PEAR_Error')) {
                 $body = sprintf(_("Unable to access %s's calendar:"), $resource) . "\n\n" . $summary;
                 $subject = sprintf(_("Error processing \"%s\""), $summary);
@@ -702,10 +718,6 @@ class Kolab_Resource
                 $subject = sprintf(_("%s has been cancelled"), $summary);
             }
 
-            /* Switch back to utf-8 */
-            $body = String::convertCharset($body, 'iso-8859-1');
-            $subject = String::convertCharset($subject, 'iso-8859-1');
-
             Horde::logMessage(sprintf('Sending confirmation of cancelation to %s', $organiser),
                               __FILE__, __LINE__, PEAR_LOG_WARNING);
 
@@ -716,8 +728,6 @@ class Kolab_Resource
 
             // Build the reply headers.
             $msg_headers = &new MIME_Headers();
-            $msg_headers->addReceivedHeader();
-            $msg_headers->addMessageIdHeader();
             $msg_headers->addHeader('Date', date('r'));
             $msg_headers->addHeader('From', $resource);
             $msg_headers->addHeader('To', $organiser);
@@ -870,7 +880,7 @@ class Kolab_Resource
         @$http->sendRequest();
         if ($http->getResponseCode() != 200) {
             return PEAR::raiseError(sprintf('Unable to retrieve free/busy information for %s',
-                                            $email), 
+                                            $email),
                                     OUT_LOG | EX_UNAVAILABLE);
         }
         $vfb_text = $http->getResponseBody();
@@ -951,7 +961,7 @@ class Kolab_Resource
         //     for( $i = 0; $i < count($attendees); $i++ ) {
         //       $attendee = preg_replace('/^mailto:\s*/i', '', $attendees[$i]);
         //       if ($attendee != $resource) {
-        // 	continue;
+        //	continue;
         //       }
         //       $params = $params[$i];
         //       break;
@@ -1018,8 +1028,6 @@ class Kolab_Resource
 
         // Build the reply headers.
         $msg_headers = &new MIME_Headers();
-        $msg_headers->addReceivedHeader();
-        $msg_headers->addMessageIdHeader();
         $msg_headers->addHeader('Date', date('r'));
         $msg_headers->addHeader('From', "$cn <$resource>");
         $msg_headers->addHeader('To', $organiser);
